// @flow
import Row from './Row';
import type { Cell } from './type';

/**
 * 区域布局服务
 *
 * @export
 * @class ColumnLayout
 */
export default class ColumnLayout {
  flex: number;
  width: number;
  cells: Cell[];
  point: {
    x: number,
    y: number,
  };
  rows: Row[];
  /**
   * Creates an instance of ColumnLayout.
   * @param {any} flex  区域的flex
   * @param {any} width  区域的宽度
   * @param {any} cells  区域中的区块信息
   * @param {any} point  点的坐标
   * @memberof ColumnLayout
   */
  constructor(
    flex: number,
    width: number,
    cells: Cell[],
    point: {
      x: number,
      y: number,
    },
  ) {
    this.flex = flex;
    this.width = width;
    this.cells = cells;
    this.rows = [];
    this.point = point;
    this.layout();
  }

  /**
   * 处理布局
   *
   * @memberof ColumnLayout
   */
  layout() {
    for (const cell of this.cells) {
      const row = this.getRow(cell);
      row.addCell(cell);
    }
  }

  /**
   * 获取当前行
   *
   * @returns
   * @memberof ColumnLayout
   */
  getCurrentRow() {
    const rows = this.rows;
    if (rows.length === 0) {
      const width = this.width;
      const flex = this.flex;
      rows.push(new Row(width, flex, 0));
      return rows[0];
    }
    return rows[rows.length - 1];
  }

  /**
   * 获取能够容纳cell的行
   *
   * @param {any} cell
   * @returns
   * @memberof ColumnLayout
   */
  getRow(cell: Cell) {
    const row = this.getCurrentRow();
    if (row.canHold(cell)) {
      return row;
    }
    const width = this.width;
    const flex = this.flex;
    const nextRow = new Row(width, flex, row.y + row.height);
    this.rows.push(nextRow);
    return nextRow;
  }

  getOverRow(y: number) {
    let rowPos = 0;
    for (let i = 0; i < this.rows.length; i++) {
      const row = this.rows[i];
      rowPos += row.height;
      if (y <= rowPos) {
        return i;
      }
    }

    if (y > 0) {
      return this.rows.length;
    }
    return -1;
  }

  getOverCell(x: number, y: number): Cell {
    const rowIndex = this.getOverRow(y);
    if (rowIndex === -1) {
      return undefined;
    }
    if (rowIndex === this.rows.length) {
      return this.cells[this.cells.length - 1];
    }
    const row = this.rows[rowIndex];
    let cellPos = 0;
    for (const cell of row.cells) {
      cellPos += this.getCellWidth(cell);
      if (x <= cellPos) {
        return cell;
      }
    }
    return row.cells[row.cells.length - 1];
  }

  getCellWidth(cell: Cell) {
    return cell.flex ? cell.flex * (this.width / this.flex) : this.width;
  }

  isAtBefore(rowIndex: number, x: number) {
    const row = this.rows[rowIndex];
    let start = 0;
    let end = 0;
    for (const cell of row.cells) {
      const width = this.getCellWidth(cell);
      end += width;
      if (x <= end) {
        return (start + end) / 2 > x;
      }
      start += width;
    }
    return false;
  }

  getCellIndex(cell: Cell) {
    return this.cells.indexOf(cell);
  }

  getBeforeCellId(cell: Cell) {
    const cellIndex = this.getCellIndex(cell);
    if (cellIndex <= 0) {
      return undefined;
    }
    return this.cells[cellIndex - 1].id;
  }

  getNextCellId(cell: Cell) {
    const cellIndex = this.getCellIndex(cell);
    if (cellIndex === this.cells.length - 1) {
      return undefined;
    }
    return this.cells[cellIndex + 1].id;
  }

  /**
   * 获取已知点的位置
   *
   * @returns
   * @memberof ColumnLayout
   */
  getPointPosition() {
    const rowIndex = this.getOverRow(this.point.y);
    const cell = this.getOverCell(this.point.x, this.point.y);
    let isAtBefore = false;

    if (rowIndex === -1) {
      return undefined;
    }

    if (rowIndex < this.rows.length) {
      isAtBefore = this.isAtBefore(rowIndex, this.point.x);
    }

    const result = cell
      ? {
          before: isAtBefore ? this.getBeforeCellId(cell) : cell.id,
          next: isAtBefore ? cell.id : this.getNextCellId(cell),
          over: cell.id,
          isAtBefore,
        }
      : {};

    return result;
  }
}
