import update from 'immutability-helper';

/**
 * 处理首页布局模型数据
 *
 * @export
 * @class IndexLayoutModal
 */
export default class IndexLayoutModal {
  /**
   * Creates an instance of IndexLayoutModal.
   * @param {any} cells   区块配置
   * @memberof IndexLayoutModal
   */
  constructor(cells) {
    this.cells = cells;
    this.columns = [];
  }

  setLayout(columns) {
    this.columns = columns.map((column, i) => ({
      ...column,
      id: `${i + 1}`,
      cells: this.cells.filter(cell => column.cells.includes(cell.id)),
    }));
  }

  moveCellToBeforeOf(movedCellId: string, overCellId: string) {
    const columnOfOverCell = this.getColumnOfCell(overCellId);
    const columnOfMovedCell = this.getColumnOfCell(movedCellId);
    const movedCellIndex = columnOfMovedCell
      ? columnOfMovedCell.cells.findIndex(cell => cell.id === movedCellId)
      : 0;
    const overCellIndex = columnOfOverCell.cells.findIndex(
      cell => cell.id === overCellId,
    );

    if (
      columnOfMovedCell === columnOfOverCell &&
      movedCellIndex + 1 === overCellIndex
    ) {
      return;
    }

    let cells = columnOfMovedCell
      ? columnOfMovedCell.cells
      : columnOfOverCell.cells;
    if (columnOfMovedCell === columnOfOverCell) {
      cells = update(cells, {
        $splice: [[movedCellIndex, 1]],
      });
    }
    cells = update(cells, {
      $splice: [[overCellIndex, 0, this.getCellById(movedCellId)]],
    });

    this.updateCellsOfColumn(columnOfOverCell, cells);
  }

  moveCellToAfterOf(movedCellId: string, overCellId: string) {
    const columnOfOverCell = this.getColumnOfCell(overCellId);
    const columnOfMovedCell = this.getColumnOfCell(movedCellId);
    const movedCellIndex = columnOfMovedCell
      ? columnOfMovedCell.cells.findIndex(cell => cell.id === movedCellId)
      : 0;
    const overCellIndex = columnOfOverCell.cells.findIndex(
      cell => cell.id === overCellId,
    );

    let cells = columnOfMovedCell
      ? columnOfMovedCell.cells
      : columnOfOverCell.cells;
    if (columnOfMovedCell === columnOfOverCell) {
      cells = update(cells, {
        $splice: [[movedCellIndex, 1]],
      });
    }
    cells = update(cells, {
      $splice: [[overCellIndex + 1, 0, this.getCellById(movedCellId)]],
    });

    this.updateCellsOfColumn(columnOfOverCell, cells);
  }

  getColumnIndex(column) {
    return this.columns.indexOf(column);
  }

  getCellById(cellId) {
    return this.cells.find(cell => cell.id === cellId);
  }

  getColumnOfCell(cellId: string) {
    return this.columns.find(
      column => column.cells.findIndex(cell => cell.id === cellId) >= 0,
    );
  }

  removeCellFromColumn(cellId: string, column) {
    this.updateCellsOfColumn(
      column,
      update(column.cells, {
        $splice: [[column.cells.findIndex(cell => cell.id === cellId), 1]],
      }),
    );
  }

  updateCellsOfColumn(column, cells) {
    this.columns = update(this.columns, {
      $splice: [
        [
          this.getColumnIndex(column),
          1,
          {
            ...column,
            cells,
          },
        ],
      ],
    });
  }

  moveCell(movedCellId, relativePosition) {
    if (relativePosition === {} || relativePosition.over === movedCellId) {
      return;
    }

    const columnOfMovedCell = this.getColumnOfCell(movedCellId);
    const columnOfOverCell = this.columns.find(
      column => column.id === relativePosition.column,
    );

    if (columnOfMovedCell !== columnOfOverCell) {
      this.removeCellFromColumn(movedCellId, columnOfMovedCell);
    }

    if (columnOfOverCell.cells.length === 0) {
      this.updateCellsOfColumn(columnOfOverCell, [
        this.getCellById(movedCellId),
      ]);
      return;
    }

    if (relativePosition.isAtBefore) {
      this.moveCellToBeforeOf(movedCellId, relativePosition.over);
    } else {
      this.moveCellToAfterOf(movedCellId, relativePosition.over);
    }
  }

  get modal() {
    return this.columns;
  }
}
